<?php

/**
 * エラーを捕捉する例
 */

error_reporting(E_ALL);
register_shutdown_function('shutdown');
set_error_handler('exception_error_handler');
set_exception_handler('exception_handler');

// hoge();
$a = 1;
$b = [];
echo $a + $b;
exit;


/**
 * エラーハンドラー関数
 *
 * エラーを例外に変換する
 *
 * @see http://php.net/manual/class.errorexception.php#errorex
 */
function exception_error_handler($severity, $message, $file, $line) {
    if (!(error_reporting() & $severity)) {
        // このエラーコードが error_reporting に含まれていない場合
        return;
    }
    throw new ErrorException($message, 0, $severity, $file, $line);
}

/**
 * 例外ハンドラー関数
 *
 * try-catchで捕捉されない例外はここで処理されます
 *
 * @param \Exception|\Throwable $e
 */
function exception_handler($e)
{
    // 以下の例外を表示する処理は自作するよりも whoops を使うと
    // 簡単でかっこいいエラー表示画面を表示できます。
    // ただし、間違った設定によって利用者にPHPコードが露出しないよう気をつけてください。
    // @see https://github.com/filp/whoops

    printf("[%s]".PHP_EOL, $e->getMessage());
    printf("File: %s:%d".PHP_EOL, $e->getFile(), $e->getLine());
    echo "Trace:";
    var_dump($e->getTrace());

    // 必要があればログに記録する
    // mylogger_exception($e);
}

/**
 * シャットダウンハンドラー関数
 *
 * PHPスクリプトの終了時に捕捉できなかった致命的エラーがあればログに記録などを行う
 */
function shutdown()
{
    // 最後に発生したエラーを取得する
    $error = error_get_last();

    // エラーが起こらなければ何もしない
    if ($error === null) {
        return;
    }

    // 必要があればログに記録する
    // mylogger_error($error);
}
