<?php

/**
 * explodeの例
 *
 * これは、掲示板のログのような文字列データを整形するプログラムの例です
 *
 * 現在は「Re:ゼロから始める異世界生活おもしろかったです」など、
 * いくつかの箇所が正しく表示できない状態です。
 *
 * バグを修正してみてください。
 *
 * ## 仕様
 *
 * ログファイルにはいくつもの「メッセージ」が含まれ、 "-----\n" で分割される。
 * メッセージは「ヘッダ」と「ボディ」に分かれており、 "\n\n" で分割される。
 * ヘッダは「ラベル」と「値」に分かれており ":" で分割される。
 *
 * (注意：仕様には記述の不備があります…)
 */

$log = <<<EOM
From:X:太郎
Subject:はいふりおもしろかったです
Date:2016-08-10

こんばんは、X:太郎と申します。
はいふり最高でしたね〜〜〜〜〜〜〜〜〜〜〜〜〜！！
-----
From:次郎
Subject:Re:ゼロから始める異世界生活おもしろかったです
Date:2016-08-10

おばんです、次郎です。

とても良かったです。
次章もとても楽しみです

EOM;

$messages = [];

// ログファイルをプログラムとして扱いやすいように整形
foreach (explode("-----\n", $log) as $msg) {
    $header = [];
    list($headers_str, $message_body) = explode("\n\n", $msg);
    foreach (explode("\n", $headers_str) as $line) {
        // この辺りが怪しい
        list($label, $value) = explode(":", $line);
        $header[strtolower($label)] = $value;
    }

    $messages[] = [
        'header' => $header,
        'body'   => $message_body,
    ];
}

if (false) {
    echo "Original:";
    var_dump($log);
    echo PHP_EOL;
}

echo "Format:", PHP_EOL;
// 配列の中身を順に画面に出力していく
foreach ($messages as $message) {
    $body_split = explode("\n", trim($message['body']));
    // 一番長い行を調べる
    $width = max(
        array_merge(
            array_map(
                function ($l) { return mb_strwidth($l) + 9; },
                $message['header']),
            array_map('mb_strwidth', $body_split)));
    echo '+-', str_repeat('-', $width), '-+', PHP_EOL;

    $padding = function ($b, $d = 0) use ($width) {
        return str_repeat(' ', ($width) - $d - mb_strwidth($b));
    };

    printf('| [件名]   %s%s |'.PHP_EOL, $message['header']['subject'], $padding($message['header']['subject'], 9));
    printf('| [投稿者] %s%s |'.PHP_EOL, $message['header']['from'], $padding($message['header']['from'], 9));

    // メッセージ本文を出力する
    foreach ($body_split as $body_line) {
        echo '| ', $body_line, $padding($body_line), ' |', PHP_EOL;
    }

    echo '+-', str_repeat('-', $width), '-+', PHP_EOL;
}
